'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fbWatchman = require('fb-watchman');

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var WatchmanConnector = function (_EventEmitter) {
  _inherits(WatchmanConnector, _EventEmitter);

  function WatchmanConnector() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? { aggregateTimeout: 200, projectPath: '' } : arguments[0];

    _classCallCheck(this, WatchmanConnector);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(WatchmanConnector).call(this));

    _this.aggregatedChanges = [];
    _this.connected = false;
    _this.fileTimes = {};
    _this.paused = true;
    _this.timeoutRef = 0;
    _this.initialScan = true;
    _this.initialScanQueue = new Set();

    _this._onSubscription = function (resp) {
      if (resp.subscription === 'webpack_subscription') {
        resp.files.forEach(function (file) {
          var filePath = _path2.default.join(_this.options.projectPath, file.name);
          var mtime = !file.exists ? null : +file.mtime_ms;

          _this._setFileTime(filePath, mtime);

          if (_this.initialScan && mtime) {
            _this.initialScanQueue.add({ name: filePath, mtime: mtime });
          }

          if (_this.paused || !file.exists) return;

          _this._handleEvents(filePath, mtime);
        });
      }
    };

    _this._onTimeout = function () {
      _this.timeoutRef = 0;
      var changes = _this.aggregatedChanges;
      _this.aggregatedChanges = [];

      _this.emit('aggregated', changes);
    };

    if (!options.projectPath) throw new Error('projectPath is missing for WatchmanPlugin');

    _this.options = options;
    return _this;
  }

  _createClass(WatchmanConnector, [{
    key: 'watch',
    value: function watch(files, dirs) {
      var _this2 = this;

      this.paused = false;

      if (this.connected) return;

      var allFiles = files.concat(dirs);
      var client = this._getClientInstance();

      client.capabilityCheck({ optional: [], required: ['cmd-watch-project', 'relative_root'] }, function (capabilityErr) {
        if (capabilityErr) throw capabilityErr;

        // Initiate the watch
        client.command(['watch-project', _this2.options.projectPath], function (watchError, watchResponse) {
          if (watchError) throw watchError;

          if (watchResponse.warning) {
            console.log('warning: ', watchResponse.warning); // eslint-disable-line no-console
          }

          client.command(['clock', watchResponse.watch], function (clockError, clockResponse) {
            if (clockError) throw clockError;

            var sub = {
              expression: ['allof', ['name', allFiles.map(function (file) {
                return _path2.default.relative(_this2.options.projectPath, file);
              }), 'wholename']],
              fields: ['name', 'mtime_ms', 'exists'],
              since: clockResponse.clock,
              relative_root: watchResponse.relative_path
            };

            client.on('subscription', _this2._onSubscription);

            client.command(['subscribe', watchResponse.watch, 'webpack_subscription', sub], function (subscribeError) {
              if (subscribeError) throw subscribeError;
            });
          });
        });
      });

      this._doInitialScan(allFiles);
    }
  }, {
    key: 'getTimes',
    value: function getTimes() {
      return this.fileTimes;
    }
  }, {
    key: 'close',
    value: function close() {
      this.paused = true;
      if (this.timeoutRef) clearTimeout(this.timeoutRef);

      // Create variable for flow
      var client = this.client;
      if (client) {
        client.removeListener('subscription', this._onSubscription);
        client.command(['unsubscribe', this.options.projectPath, 'webpack_subscription']);
        client.end();
        this.client = null;
      }
    }
  }, {
    key: 'pause',
    value: function pause() {
      this.paused = true;
      if (this.timeoutRef) clearTimeout(this.timeoutRef);
    }
  }, {
    key: '_setFileTime',
    value: function _setFileTime(file, mtime) {
      this.fileTimes[file] = mtime;
    }
  }, {
    key: '_handleEvents',
    value: function _handleEvents(filePath, mtime) {
      this.emit('change', filePath, mtime);

      this._handleAggregated(filePath);
    }
  }, {
    key: '_handleAggregated',
    value: function _handleAggregated(file) {
      if (this.timeoutRef) clearTimeout(this.timeoutRef);

      if (this.aggregatedChanges.indexOf(file) < 0) {
        this.aggregatedChanges.push(file);
      }

      this.timeoutRef = setTimeout(this._onTimeout, this.options.aggregateTimeout);
    }
  }, {
    key: '_getClientInstance',
    value: function _getClientInstance() {
      var _this3 = this;

      if (!this.client) {
        // Create variable for flow
        var client = new _fbWatchman.Client();
        client.on('connect', function () {
          _this3.connected = true;
        });
        client.on('end', function () {
          _this3.connected = false;
        });

        this.client = client;
      }

      return this.client;
    }
  }, {
    key: '_doInitialScan',
    value: function _doInitialScan(files) {
      var _this4 = this;

      _async2.default.eachLimit(files, 100, function (file, callback) {
        _fs2.default.stat(file, function (err, stat) {
          if (err) {
            callback(err);
            return;
          }

          _this4._setFileTime(file, +stat.mtime);
          callback();
        });
      }, function () {
        _this4.initialScan = false;

        if (_this4.initialScanQueue.size > 0) {
          var file = Array.from(_this4.initialScanQueue)[_this4.initialScanQueue.size - 1];
          _this4._handleEvents(file.name, file.mtime);
        }

        _this4.initialScanQueue.clear();
      });
    }
  }]);

  return WatchmanConnector;
}(_events.EventEmitter);

exports.default = WatchmanConnector;